@@ -107,23 +107,28 @@ pub struct CliArgs {
     #[clap(short = 'q', long = "qrcode")]
     pub qrcode: bool,
 
-    /// Enable file uploading
-    #[clap(short = 'u', long = "upload-files")]
-    pub file_upload: bool,
+    /// Enable file uploading (and optionally specify for which directory)
+    #[clap(short = 'u', long = "upload-files", value_hint = ValueHint::FilePath, min_values = 0)]
+    pub allowed_upload_dir: Option<Vec<PathBuf>>,
 
     /// Enable creating directories
-    #[clap(short = 'U', long = "mkdir", requires = "file-upload")]
+    #[clap(short = 'U', long = "mkdir", requires = "allowed-upload-dir")]
     pub mkdir_enabled: bool,
 
     /// Specify uploadable media types
-    #[clap(arg_enum, short = 'm', long = "media-type", requires = "file-upload")]
+    #[clap(
+        arg_enum,
+        short = 'm',
+        long = "media-type",
+        requires = "allowed-upload-dir"
+    )]
     pub media_type: Option<Vec<MediaType>>,
 
     /// Directly specify the uploadable media type expression
     #[clap(
         short = 'M',
         long = "raw-media-type",
-        requires = "file-upload",
+        requires = "allowed-upload-dir",
         conflicts_with = "media-type"
     )]
     pub media_type_raw: Option<String>,
@@ -203,7 +208,7 @@ pub struct CliArgs {
     pub readme: bool,
 }
 
-/// Checks wether an interface is valid, i.e. it can be parsed into an IP address
+/// Checks whether an interface is valid, i.e. it can be parsed into an IP address
 fn parse_interface(src: &str) -> Result<IpAddr, std::net::AddrParseError> {
     src.parse::<IpAddr>()
 }
